import { diffList } from "./array-diff.js";
export function getFirstSnapshot(parent) {
    const elements = getFlipableElements(parent);
    return elements.map(toSnapshotElement);
}
export function animateChange(parent, firstSnapshot) {
    // TODO stop prev animations
    const flips = getFlips(parent, firstSnapshot);
    const { added, moved } = groupFlips(flips);
    const removeDuration = 0;
    const moveDuration = fullStaggerDuration(moved.length, config.moveDuration);
    const addDuration = fullStaggerDuration(added.length, config.addDuration);
    moved.forEach((group, groupIndex) => {
        group.forEach((flip) => {
            const { first, last, element } = flip;
            const delay = removeDuration +
                staggerDelay(groupIndex, moved.length, moveDuration, config.moveDuration);
            animateMove(element, first, last, delay);
        });
    });
    added.forEach((group, groupIndex) => {
        group.forEach((flip) => {
            const { first, last, element } = flip;
            const delay = removeDuration +
                moveDuration +
                staggerDelay(groupIndex, added.length, addDuration, config.addDuration);
            animateAdd(element, last, delay);
        });
    });
}
function animateMove(element, first, last, delay) {
    const dx = first.x - last.x;
    const dy = first.y - last.y;
    element.animate({
        // opacity: [first.opacity, last.opacity],
        transform: [`translate(${dx}px, ${dy}px)`, "translate(0, 0)"],
        color: [first.color, last.color],
    }, {
        duration: config.moveDuration,
        easing: "ease-in-out",
        fill: "backwards",
        delay,
    });
}
function animateAdd(element, last, delay) {
    element.animate({ opacity: [0, 1] }, {
        duration: config.addDuration,
        fill: "both",
        easing: "ease-out",
        delay,
    });
}
function groupFlips(flips) {
    const added = [];
    const forwards = [];
    const backwards = [];
    let lastBin = null;
    flips.forEach((flip) => {
        const { first, last } = flip;
        let bin = null;
        if (!first) {
            bin = added;
        }
        else if (first.x === last.x && first.y === last.y) {
            // unchanged
            bin = null;
        }
        else {
            const dx = first.x - last.x;
            const dy = first.y - last.y;
            const bwd = dy > 0 || (dy == 0 && dx > 0);
            bin = bwd ? backwards : forwards;
        }
        if (bin && bin !== lastBin) {
            bin.push([flip]);
        }
        else if (bin) {
            bin[bin.length - 1].push(flip);
        }
        lastBin = bin;
    });
    forwards.reverse();
    const moved = [...backwards, ...forwards];
    return { added, moved };
}
// ---
const config = {
    removeDuration: 100,
    moveDuration: 250,
    addDuration: 200,
};
function fullStaggerDuration(count, singleDuration) {
    if (count === 0)
        return 0;
    return 2 * singleDuration * (1 - 1 / (1 + count));
    // return 1.5 * singleDuration - 1 / (1 + count)
}
function staggerDelay(i, n, duration, singleDuration) {
    if (i === 0)
        return 0;
    const max = duration - singleDuration;
    return (i / (n - 1)) * max;
}
// ---
function getFlips(parent, firstSnapshot) {
    const elements = getFlipableElements(parent);
    // TODO stop prev animations
    const flips = elements.map((element) => ({
        element,
        first: null,
        last: toSnapshotElement(element),
    }));
    const firstContent = firstSnapshot.map((e) => e.content);
    const lastContent = flips.map((e) => e.last.content);
    diffList(firstContent, lastContent).forEach(([firstIndex, lastIndex]) => {
        flips[lastIndex].first = firstSnapshot[firstIndex];
    });
    return flips;
}
function getFlipableElements(parent) {
    // TODO check browser support (Firefox missing?)
    // return Array.from(parent.querySelectorAll(":not(:has(*))"))
    return Array.from(parent.querySelectorAll("span"));
}
function toSnapshotElement(el) {
    // const { x, y } = el.getBoundingClientRect()
    let x = 0;
    let y = 0;
    let p = el;
    while (p) {
        x += p.offsetLeft;
        y += p.offsetTop;
        p = p.offsetParent;
    }
    const style = window.getComputedStyle(el);
    const color = style.color;
    const content = el.textContent;
    return { x, y, color, content };
}
