import { jsx as _jsx } from "react/jsx-runtime";
import { tokenize } from "./code-to-tokens.js";
import { isGroup, isWhitespace } from "./types.js";
import React from "react";
import { CodeClient } from "./code-client.js";
let annotationWarning = new Set();
export async function CodeContent({ codeblock, components = {}, config, ...props }) {
    const { lang, meta, value, annotations = [], parentPath } = codeblock;
    const tokens = await tokenize(value, lang || "txt", annotations, {
        theme: config.theme,
        mdxPath: parentPath,
        lines: components.Line ? true : false,
        ...config,
    });
    return (_jsx(CodeClient, { ...props, children: _jsx(TokensComponent, { tokens: tokens, components: components }) }));
}
function TokensComponent({ tokens, components, }) {
    return tokens.map((token, i) => (_jsx(TokenComponent, { token: token, components: components }, i)));
}
function TokenComponent({ token, components, }) {
    if (isGroup(token)) {
        const [[name, query, { inline } = { inline: false }], tokens] = token;
        if (components[name]) {
            return React.createElement(components[name], {
                query,
                inline,
                children: tokens.map((token, i) => (_jsx(TokenComponent, { token: token, components: components }, i))),
            });
        }
        if (name === "class") {
            return (_jsx("span", { className: query, children: tokens.map((token, i) => (_jsx(TokenComponent, { token: token, components: components }, i))) }));
        }
        if (!annotationWarning.has(name)) {
            annotationWarning.add(name);
            console.warn("Missing annotation component", name);
        }
        return (_jsx("span", { className: name, children: tokens.map((token, i) => (_jsx(TokenComponent, { token: token, components: components }, i))) }));
    }
    // if is whitespaceToken
    if (isWhitespace(token)) {
        return token;
    }
    const [content, color, style = {}] = token;
    style.color = color;
    return _jsx("span", { style: { ...style, display: "inline-block" }, children: content });
}
