import { jsx as _jsx } from "react/jsx-runtime";
import React from "react";
import { DebugHike } from "./debug-hike.js";
export function Hike({ as, hike, ...rest }) {
    if (!as || rest.debug) {
        return _jsx(DebugHike, { hike: hike });
    }
    return React.createElement(as, { hike, ...rest });
}
function StaticHike({ children }) {
    const hike = getStepsFromChildren(children);
    return slotToStaticJSX(hike);
}
function slotToStaticJSX(section) {
    return section.children.flatMap((child) => {
        if (child.type === React.Fragment && child.key) {
            const name = child.key;
            if (name === "code") {
                return section.code.map((code, i) => (_jsx("pre", { children: _jsx("code", { className: code.lang, children: code.value }) }, i)));
            }
            const sections = section[name];
            const childSlot = sections.shift();
            if (!childSlot) {
                throw new Error(`Missing slot ${name}`);
            }
            return slotToStaticJSX(childSlot);
        }
        return child;
    });
}
export function getStepsFromChildren(kids, query = "") {
    const [children, ...slotElements] = React.Children.toArray(kids);
    const slots = {};
    const code = [];
    slotElements.forEach((slotElement) => {
        const { name, query } = slotElement.props || {};
        if (name === "code") {
            code.push(getCodeBlockFromChildren(slotElement.props.children));
        }
        else {
            slots[name] = slots[name] || [];
            slots[name].push(getStepsFromChildren(slotElement.props.children, query));
        }
    });
    return {
        query,
        children: React.Children.toArray(children.props.children).map((e) => e.type === "placeholder" ? _jsx(React.Fragment, {}, e.props.name) : e),
        ...slots,
        code,
    };
}
function getCodeBlockFromChildren(children) {
    const { props } = React.Children.toArray(children)[0];
    return {
        value: props.code,
        meta: props.meta,
        lang: props.lang,
        parentPath: props.parentPath,
    };
}
