// this adds the jsx node types to Root
import "mdast-util-mdx-jsx";
import { listToTree } from "./1.remark-list-to-tree.js";
import { hydrateTree, parseCode } from "./2.hydrate-tree.js";
import { treeToAttribute } from "./3.remark-tree-to-attribute.js";
import { SKIP, visit } from "estree-util-visit";
import { moveChildrenToHikeProp } from "./4.recma-move-children.js";
import { getObjectAttribute } from "./estree.js";
export const remarkCodeHike = (config) => {
    return async (root, file) => {
        let tree = (await transformAllHikes(root, config, file));
        tree = transformAllCode(tree, config, file);
        return tree;
    };
};
export const recmaCodeHike = (config) => {
    return async (root, file) => {
        let tree = transformAllRecmaHikes(root, config);
        return tree;
    };
};
export function transformAllCode(node, config, file) {
    const mdxPath = (file === null || file === void 0 ? void 0 : file.history)
        ? file.history[file.history.length - 1]
        : undefined;
    if (node.type === "code") {
        const codeblock = parseCode(node, mdxPath);
        return {
            type: "mdxJsxFlowElement",
            name: "Code",
            attributes: [
                {
                    type: "mdxJsxAttribute",
                    name: "codeblock",
                    value: getObjectAttribute(codeblock),
                },
            ],
            children: [],
        };
    }
    if ("children" in node && node.children.length > 0) {
        // @ts-ignore
        node.children = node.children.map((child) => transformAllCode(child, config, file));
    }
    return node;
}
export async function transformAllHikes(node, config, file) {
    const mdxPath = (file === null || file === void 0 ? void 0 : file.history) ? file.history[file.history.length - 1] : null;
    if (node.type === "mdxJsxFlowElement" && node.name === "Hike") {
        return await transformRemarkHike(node, mdxPath);
    }
    if ("children" in node && node.children.length > 0) {
        // @ts-ignore
        node.children = await Promise.all(node.children.map((child) => transformAllHikes(child, config, file)));
    }
    return node;
}
async function transformRemarkHike(node, mdxPath) {
    var _a;
    const prefix = "!";
    const tree = listToTree(node, prefix);
    const hydratedTree = hydrateTree(tree, mdxPath);
    const { children, attributes } = treeToAttribute(hydratedTree);
    node.children = children;
    node.attributes.push(...attributes);
    const asAttribute = node.attributes.find((a) => a.name === "as");
    const debug = node.attributes.find((a) => a.name === "debug");
    node.attributes = node.attributes.filter((a) => a.name !== "as");
    if (asAttribute && !debug) {
        node.name = ((_a = asAttribute === null || asAttribute === void 0 ? void 0 : asAttribute.value) === null || _a === void 0 ? void 0 : _a.value) || "Hike";
    }
    return node;
}
export function transformAllRecmaHikes(tree, config) {
    visit(tree, (node) => {
        var _a, _b;
        if ((node === null || node === void 0 ? void 0 : node.type) === "JSXElement" &&
            ((_b = (_a = node === null || node === void 0 ? void 0 : node.openingElement) === null || _a === void 0 ? void 0 : _a.attributes) === null || _b === void 0 ? void 0 : _b.some((a) => { var _a; return ((_a = a === null || a === void 0 ? void 0 : a.name) === null || _a === void 0 ? void 0 : _a.name) === "__hike"; }))) {
            transformRecmaHike(node);
            return SKIP;
        }
    });
    return tree;
}
function transformRecmaHike(node) {
    moveChildrenToHikeProp(node);
}
